mod body;

use body::*;

const FLAGDELIMITER: u8 = 0x7E;

#[allow(dead_code)]
const PropsSizeMask: u16 = 0x01FF;
const PropsSubcontractMask: u16 = 0x3000;
const PropsReservedMask: u16 = 0xC000;
const PropsDataEncryptionMask: u16 = 0x0E00;

pub fn parse_inbound_msg(rawdata: &mut Vec<u8>) -> Message {
    let mut msg: Message = Message::default();

    msg.parse_header(rawdata);
    msg.check(rawdata);

    msg
}

#[derive(Default, Debug)]
pub struct Message {
    pub header: MessageHeader,
    pub body: Vec<u8>,
    pub checksum: u8,
    valid: bool,
}

impl Message {
    pub fn parse_header(&mut self, rawdata: &mut Vec<u8>) {
        let mut data = rawdata.clone().into_iter();
        let first_byte = data.next().unwrap();
        if first_byte == FLAGDELIMITER {
            println!("first flag ok");
        };
        self.header.id = ((data.next().unwrap() as u16) << 8) + data.next().unwrap() as u16;
        self.header.properties = ((data.next().unwrap() as u16) << 8) + data.next().unwrap() as u16;
        for i in 0..self.header.terminal_id.len() {
            self.header.terminal_id[i] = data.next().unwrap();
        }
        self.parse_properties();

        self.header.serial_number =
            ((data.next().unwrap() as u16) << 8) + data.next().unwrap() as u16;

        for _ in 0..self.header.properties as usize {
            self.body.push(data.next().unwrap());
        }

        self.checksum = data.next().unwrap();

        if data.next().unwrap() == FLAGDELIMITER {
            println!("end flag ok");
        } else {
            println!("error with endflag");
        }
    }

    pub fn parse_properties(&mut self) {
        self.header.bodylength = (self.header.properties & PropsSizeMask) as u8;
    }

    pub fn check(&mut self, rawdata: &mut Vec<u8>) {
        let mut data = rawdata.into_iter();
        data.next().unwrap();
        let mut check: u8 = 0;

        loop {
            let b = data.next().unwrap();
            if *b == FLAGDELIMITER {
                break;
            }
            check = *b ^ check;
        }

        self.validate(check);
    }

    fn validate(&mut self, check: u8) {
        println!("{}", BodyType::TerminalHeartbeat as u16);
        self.valid = check == 0;
    }

    pub fn is_valid(&self) -> bool {
        self.valid
    }
}

#[derive(Default, Debug)]
pub struct MessageHeader {
    pub id: u16,
    properties: u16,
    pub bodylength: u8,
    pub encryption: bool,
    pub terminal_id: [u8; 6],
    pub serial_number: u16,
}
